<div class="header-wrapper">
    <div class="header-container">
        <a href="/" class="header-logo">Nexom</a>
        <a href="https://github.com/ait913/Nexom" class="header-link", title="to GitHub Repository">
            <img src="/static/github.png" alt="GitHub Link">
        </a>
    </div>
</div>
<div class="header-speacer"></div>